==> heptapod-rails-0.16.1.tar.gz: heptapod-rails-0.16.1/lib/backup/namespace.rb <==
require 'find'
require 'pathname'
require 'yaml'
require 'rake'

module Backup
  class Namespace

    def dump
      begin
        prepare
        backup_namespaces_hgrc
      ensure
        FileUtils.rm_rf(backup_dir)
      end
    end

    def backup_namespaces_hgrc
      # Backup::Files does that with an rsync, that's probably indeed
      # much faster than looping in Ruby (using the right exclude options)
      # provided it's guaranteed to be installed.
      root = repositories_root
      Find.find(root) do |path|
        basename = File.basename(path)
        if FileTest.directory?(path)
          if basename.end_with?('.hg') || basename.end_with?('.git')
            Find.prune
          else
            next
          end
        elsif basename == 'hgrc'
          rel_dir = Pathname(path).parent.relative_path_from(root)
          FileUtils.mkdir_p(File.join(backup_dir, rel_dir))
          FileUtils.copy(path, File.join(backup_dir, rel_dir, basename))
        end
      end
      # In case there's no hgrc, the tarball will not be considered empty
      # (that fails), it will still have './'
      puts "Calling tar czf #{backup_tarball} -C #{backup_dir} ."
      output, status = Gitlab::Popen.popen(
                ['tar', 'czf', backup_tarball, '-C', backup_dir, '.'])
      puts "output=#{output}, status=#{status}"
      unless status.zero?
        puts output
        abort 'Backup of namespaces failed'
      end

      # prepare() is supposed to guarantee we've been the ones to create it
      FileUtils.rm_f(backup_dir)
    end

    def restore
      output, status = Gitlab::Popen.popen(
                ['tar', 'xzf', backup_tarball, '-C', repositories_root.to_s])
      unless status.zero?
        puts output
        abort 'Restore of namespaces failed'
      end
    end

    protected

    def repositories_root
      storages = Gitlab.config.repositories.storages.values
      if storages.length > 1
        puts "Sorry, Heptapod supports one single storage only."
        abort
      end
      Pathname(storages[0].hg_disk_path).realpath
    end

    def backup_tarball
      File.join(Gitlab.config.backup.path, 'namespaces.tar.gz')
    end

    def backup_dir
      File.join(Gitlab.config.backup.path, 'namespaces')
    end

    def prepare
      FileUtils.rm_f(backup_tarball)
      # Ensure the parent dir exists
      # TODO taken from GitLab's code, but it's data duplication, should
      # use a parent dir method instead
      FileUtils.mkdir_p(Gitlab.config.backup.path)
      # Fail if somebody raced to create backup_dir before us
      FileUtils.mkdir(backup_dir, mode: 0700)
    end

    def silent
      { err: '/dev/null', out: '/dev/null' }
    end
  end
end

==> heptapod-rails-0.21.3.tar.gz: heptapod-rails-0.21.3/lib/backup/namespace.rb <==
# frozen_string_literal: true
require 'find'
require 'pathname'
require 'yaml'
require 'rake'

module Backup
  class Namespace
    def dump
      prepare
      backup_namespaces_hgrc
    ensure
      FileUtils.rm_rf(backup_dir)
    end

    def backup_namespaces_hgrc
      # Backup::Files does that with an rsync, that's probably indeed
      # much faster than looping in Ruby (using the right exclude options)
      # provided it's guaranteed to be installed.
      root = repositories_root
      Find.find(root) do |path|
        basename = File.basename(path)
        if FileTest.directory?(path)
          if basename.end_with?('.hg', '.git')
            Find.prune
          else
            next
          end
        elsif basename == 'hgrc'
          rel_dir = Pathname(path).parent.relative_path_from(root)
          FileUtils.mkdir_p(File.join(backup_dir, rel_dir))
          FileUtils.copy(path, File.join(backup_dir, rel_dir, basename))
        end
      end
      # In case there's no hgrc, the tarball will not be considered empty
      # (that fails), it will still have './'
      puts "Calling tar czf #{backup_tarball} -C #{backup_dir} ."
      output, status = Gitlab::Popen.popen(
        ['tar', 'czf', backup_tarball, '-C', backup_dir, '.'])
      puts "output=#{output}, status=#{status}"
      unless status == 0
        puts output
        abort 'Backup of namespaces failed'
      end

      # prepare() is supposed to guarantee we've been the ones to create it
      FileUtils.rm_f(backup_dir)
    end

    def restore
      output, status = Gitlab::Popen.popen(
        ['tar', 'xzf', backup_tarball, '-C', repositories_root.to_s])

      unless status == 0
        puts output
        abort 'Restore of namespaces failed'
      end
    end

    protected

    def repositories_root
      storages = Gitlab.config.repositories.storages.values
      if storages.length > 1
        puts "Sorry, Heptapod supports one single storage only."
        abort
      end

      Pathname(storages[0].hg_disk_path).realpath
    end

    def backup_tarball
      File.join(Gitlab.config.backup.path, 'namespaces.tar.gz')
    end

    def backup_dir
      File.join(Gitlab.config.backup.path, 'namespaces')
    end

    def prepare
      FileUtils.rm_f(backup_tarball)
      # Ensure the parent dir exists
      # TODO taken from GitLab's code, but it's data duplication, should
      # use a parent dir method instead
      FileUtils.mkdir_p(Gitlab.config.backup.path)
      # Fail if somebody raced to create backup_dir before us
      FileUtils.mkdir(backup_dir, mode: 0700)
    end

    def silent
      { err: '/dev/null', out: '/dev/null' }
    end
  end
end
